<?php
namespace Models;

// модель для чеков
class Egais {
	public $db, $f3, $web, $url_egais, $api_data;

	public function __construct($db, $f3) {
		$this->db=$db;
		$this->f3=$f3;
		$settings=new \DB\SQL\Mapper($this->db, 'settings');
		$settings->load(array('name=?', 'url_egais'), array('limit'=>1));
		$this->url_egais=$settings->value;
		$this->web=\Web::instance();
		$this->api_data=\Models\Controller::get_api_config();
	}

	// Верификация марки
	function mark_check($code) {
		$data=array(
			'code'=>$code
		);
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		),
    		'content'=>$data
		);
		$request=$this->web->request($this->url_egais.'/api/mark/check', $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($body['owner']===true) {
				return array(
					'success'=>true,
					'code'=>$mark
				);
			}

			return array(
				'success'=>false,
				'txt'=>'products_did_not_pass_inspection'
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Информация о выданном сертификате RSA
	function get_rsa() {
		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/rsa/orginfo', $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($body['cn']) {
				return array(
					'success'=>true,
					'cn'=>$body['cn']
				);
			}
			return array(
				'success'=>false,
				'txt'=>'Не удалось получит информацию о RSA-сертификате'
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Информация об организации
	function orginfo() {
		// Узнаем номер организации, на которую выдан сертификат RSA
		$cn=$this->get_rsa();
		if(!$cn['success']) {
			echo $cn['txt'];
			exit();
		}

		$options = array(
			'method'=>'GET',
    		'headder'=>array(
    			'accept: application/json'
    		)
		);
		$request=$this->web->request($this->url_egais.'/api/rsa', $options);
		$body=json_decode($request['body'], true);
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($body['success']===true) {
				$data=array();
				// Перебираем организации, чтобы найти ту, на которую выдан сертификат
				foreach ($body['rows'] as $r) {
					if($r['Owner_ID']==$cn['cn']) {
						$data=$r;
						break;
					}
				}
				if(!$data) {
					return array(
						'success'=>false,
						'txt'=>'Не удается получить информацию об организации'
					);
				}
				return array(
					'success'=>true,
					'data'=>$data
				);
			}
			return array(
				'success'=>false,
				'txt'=>'error'
			);
		}
		if($body['message']) {
			return array(
				'success'=>false,
				'txt'=>$body['message']
			);
		}
		if($body['error']) {
			return array(
				'success'=>false,
				'txt'=>$body['error']
			);
		}
		return array(
			'success'=>false,
			'txt'=>implode(';', $request)
		);
	}

	// Чек продажи
	function cheque($change_id, $receipt_id, $lines, $return=false) {
		$orginfo=$this->orginfo();
		if(!$orginfo['success']) {
			echo '<p class="false">Ошибка получения данных об организации: '.$orginfo['txt'].'</p>';
			exit();
		}
		$org=$orginfo['data'];
		$xml="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		$xml.="<Cheque inn=\"".$org['INN']."\" datetime=\"".date('dmyHi')."\" kpp=\"".$org['KPP']."\" kassa=\"".$this->api_data['rm_id']."\" address=\"".$org['Fact_Address']."\" name=\"".$org['Short_Name']."\" number=\"".$receipt_id."\" shift=\"".$change_id."\">\n";
		foreach ($lines as $l) {
			if($l['egais']>0) {
				$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=$price*($l['discount']/100);
				$price-=$d_sum;
				if($return) {
					$price=-$price; // Для возвратных чеков указывается отрицательная цена
				}
				$mark=json_decode($l['mark']);
				// Объем указывается в декалитрах, 1дал=10л
				$xml.="<Bottle barcode=\"".$mark[0]."\" ean=\"".$l['barcode']."\" price=\"".number_format(round($price, 2), 2, '.', '')."\" volume=\"".number_format($l['count']/10, 4, '.', '')."\"/>\n";
			}
		}
		$xml.="</Cheque>";
		$xml_name=$this->f3->get('ROOT').'/tmp/'.(microtime(true)*10000).'.xml';
		$this->f3->write($xml_name, $xml);

		$options = array(
			'method'=>'POST',
    		'headder'=>array(
    			'accept: text/xml',
    			'Content-Type: multipart/form-data'
    		),
    		'content'=>array(
    			'xml_file'=>new \CURLFile($xml_name)
    		),
    		'timeout'=>5
		);
		$request=$this->web->request($this->url_egais.'/xml?type=Cheque', $options);
		unlink($xml_name);
		$body=json_decode($request['body'], true);
		/*$request['body'] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><A><url>https://146.120.90.148:1444?id=ab43d5e0-855c-4b54-b1d8-ddeb34d1d110&amp;dt=1503271510&amp;cn=00040218</url><sign>32B7136A7BCEAFEBE4DA92D9510C196CC42DC56260E1E2802B8C24A8C3EA75F3978EA6963F370F3165B89DBF98FD94F2F0C7F4803965B62A6616A12D8159A5D3</sign><ver>2</ver></A>";*/
		if(!is_array($body)) {
			$request['body']=str_replace("&'", "'", $request['body']);
			$xml=new \SimpleXMLElement($request['body']);
		}
		if(mb_strpos($request['headers'][0], '200')>0 || mb_strpos($request['headers'][1], '200')>0) {
			if($xml->error!='') {
				echo $xml->error;
				exit();
			}
			return array(
				'url'=>(string) $xml->url,
				'sign'=>html_entity_decode((string) $xml->sign)
			);
		}
		if(is_array($body) && $body['error']) {
			echo $body['message'];
		}
		elseif($xml->error!='') {
			echo $xml->error;
		}
		else {
			echo $request['body'];
		}
		exit();
	}
}